package oca;

import java.util.Set;

import javax.xml.namespace.QName;
import javax.xml.soap.MessageFactory;
import javax.xml.soap.Name;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPException;
import javax.xml.soap.SOAPFault;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPMessage;
import javax.xml.ws.handler.MessageContext;
import javax.xml.ws.handler.soap.SOAPHandler;
import javax.xml.ws.handler.soap.SOAPMessageContext;
import javax.xml.ws.soap.SOAPFaultException;

import com.sun.rmi.rmid.ExecPermission;
import com.sun.xml.internal.messaging.saaj.SOAPExceptionImpl;

public class ServiceAuthenticationSOAPHandler extends
		BaseHandler<SOAPMessageContext> implements
		SOAPHandler<SOAPMessageContext> {
	private final String HANDLER_NAME = "ServiceAuthenticationSOAPHandler";

	public ServiceAuthenticationSOAPHandler() {
		super();
		super.setHandlerName(HANDLER_NAME);
	}

	public boolean handleMessage(SOAPMessageContext smc) {
		boolean exit = false;
		out.println("------------------------------------");
		out.println("In SOAPHandler " + HandlerName + ":handleMessage()");

		boolean direction = ((Boolean) smc
				.get(SOAPMessageContext.MESSAGE_OUTBOUND_PROPERTY))
				.booleanValue();
		if (direction) {
			out.println("direction = outbound");
		} else {
			out.println("direction = inbound");

			// ......
			try {
				// get SOAP envelope from SOAP message
				SOAPEnvelope se = smc.getMessage().getSOAPPart().getEnvelope();

				// get the headers from envelope
				SOAPHeader sh = se.getHeader();
				if (sh == null) {
					out
							.println("--- No headers found in the input SOAP request");
					exit = false;
				} else {
					// call method to process header
					exit = processSOAPHeader(sh);

				}
			} catch (Exception ex) {
				ex.printStackTrace();
				exit = false;
			}
			// ......
			
			// Authentication failed
			if (exit == false) {
				try {
					MessageFactory messageFactory = MessageFactory.newInstance();
					SOAPMessage responseMsg = messageFactory.createMessage();
					smc.setMessage(responseMsg);
					responseMsg.getSOAPBody().addFault(
							new QName("http://oca", "AuthenticationFailedCode"),
							"Authentication failed for reason xyz");
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

		}

		out.println("Exiting SOAPHandler " + HandlerName + ":handleMessage()");
		out.println("------------------------------------");

		if (exit) {
			smc.put(AuthenticationHandlerConstants.AUTHN_STAUTS, true);
		} else {
			smc.put(AuthenticationHandlerConstants.AUTHN_STAUTS, false);
		}
		smc.setScope(AuthenticationHandlerConstants.AUTHN_STAUTS,
				MessageContext.Scope.APPLICATION);
		return exit;
	}

	private boolean processSOAPHeader(SOAPHeader sh) {
		boolean authenticated = false;

		java.util.Iterator childElems = sh.getChildElements(new QName(
				AuthenticationHandlerConstants.AUTHN_URI,
				AuthenticationHandlerConstants.AUTHN_LNAME));
		SOAPElement child;
		// iterate through child elements
		while (childElems.hasNext()) {
			child = (SOAPElement) childElems.next();
			authenticated = processSOAPHeaderInfo(child);
		}
		return authenticated;
	}

	private boolean processSOAPHeaderInfo(SOAPElement e) {

		SOAPElement child;
		QName name;
		String _id = null;
		String _password = null;

		// get an iterator on child elements of SOAP element
		java.util.Iterator childElems = e.getChildElements();

		// loop through child elements
		while (childElems.hasNext()) {
			// get next child element
			Object elem = childElems.next();
			if (elem instanceof SOAPElement) {

				child = (SOAPElement) elem;
				name = (QName) child.getElementQName();

				// get the value of id element
				if (name.getLocalPart().equals("id")) {
					out.println("\t\tid =" + child.getValue());
					_id = child.getValue();
				}

				// get the value of password element
				if (name.getLocalPart().equals("password")) {
					out.println("\t\tpassword =" + child.getValue());
					_password = child.getValue();
				}
			}
		}
		return authenticate(_id, _password);
	}

	/**
	 * In reality, we may login to a credential store.
	 */
	private boolean authenticate(String _id, String _password) {

		// client authentication information
		final String USERID = "oca";
		final String PASSWORD = "olivier";

		if (_id.equals(USERID) && _password.equals(PASSWORD)) {
			out.println("Authenticated by " + HandlerName + "!");
			return true;
		} else {
			out.println("Could not authenticate ,WRONG id or password");
		}
		return false;
	}

	public Set<QName> getHeaders() {
		return null;
	}

}
